#BEGIN COPYRIGHT
#*************************************************************************
#
# Licensed Materials - Property of IBM
# 5725-C94
# (C) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#*************************************************************************
#END COPYRIGHT

import os.path

from java.lang import System
from java.lang import Long
from com.ibm.bpe.api import UTCDate

execfile(os.path.join(System.getProperty("was.install.root"), "ProcessChoreographer/admin/bpclib.py"))
__file__ = "manageTemplates.py"


#*******************************************************************************************************
# print usage and error message, if any
#*******************************************************************************************************
def usage(errmsg = None):
  if errmsg:
    print
    print "ERROR: " + errmsg
  #endif
  print
  print "Usage: wsadmin -f " + __file__
  print "       [ -start <applicationName> ]"
  print "       [ -stop <applicationName> ]"
  print "       [ -uninstall <applicationName> [-force] ]"
  print ""
  print "Starts, stops, or uninstalls all process and task templates of a given enterprise application."
  print "Before uninstalling, the process and task templates of the enterprise application"
  print "will be stopped."
  print "CAUTION: If you specify -force, all instances of the passed enterprise application's"
  print "templates are deleted -- even if they are currently running."
  print "CAUTION: Do not use -uninstall for applications that were deployed by the Process Center."
  print "The related business-level applications are not uninstalled. You must uninstall related"
  print "business-level applications manually before you can delete the whole snapshot with the"
  print "BPMSnapshotDelete admin command."
  print ""
  print "Note that the scripting client must be connected to a running server."
  print ""
#enddef


#*******************************************************************************************************
# Change the state for all process and task templates of a given enterprise application
#*******************************************************************************************************
def changeBPCTemplateState(applicationName, message, bfmMethod, htmMethod, state):

  # 1. Find the enterprise application ----------------------------------------

  deployment = AdminConfig.getid("/Deployment:%s/" % (applicationName))
  
  if not deployment:
    raise ValueError("Application '%s' not found" % (applicationName))
  #endif
  
  scope = None
  for target in AdminUtilities.convertToList(AdminConfig.showAttribute(deployment, "deploymentTargets")):
    if AdminConfig.getObjectType(target) == "ClusteredTarget":
      scope = Scope(clusterName = AdminConfig.showAttribute(target, "name"))
    else:
      nodeName = AdminConfig.showAttribute(target, "nodeName")
      serverName = AdminConfig.showAttribute(target, "name")
      serverEntry = AdminConfig.getid("/Node:%s/ServerIndex:/ServerEntry:%s" % (nodeName, serverName))
      if AdminConfig.showAttribute(serverEntry, "serverType") == "APPLICATION_SERVER":
        scope = Scope(nodeName = nodeName, serverName = serverName)
      #endif
    #endif
  #endfor
  
  print "Application '%s' is deployed on %s." % (applicationName, scope.displayName())

  # 2. Find the BPC MBeans to connect to --------------------------------------

  bfmMBeans = scope.lookupMBean("InternalProcessContainer", 1)
  htmMBeans = scope.lookupMBean("InternalTaskManager", 1)

  # 3. Collect all process templates for the given application ----------------

  templates = getTemplatesFromConfig("ProcessComponent", applicationName, None)

  print "   Process templates:"
  if not templates:
    print "     No process templates found in '%s'." % (applicationName)
  elif not bfmMBeans:
    raise Exception("Error: Could not find a ProcessContainer MBean on %s" % (scope.displayName()))
  else:
    for template in templates:
      print "     %s '%s', valid from %s, state %s" % (message, template["templateName"], template["validFromString"], template["state"])
      for bfmMBean in bfmMBeans:
        AdminControl.invoke(bfmMBean, bfmMethod, [ template["templateName"], template["validFrom"], applicationName ] )
      #endfor
      AdminConfig.modify(template["stateManagement"], [ [ "initialState", state ] ])
    #endfor
  #endif

  # 4. Collect all task templates for the given application ----------------

  templates = getTemplatesFromConfig("TaskComponent", applicationName, None)

  print "   Task templates:"
  if not templates:
    print "     No task templates found in '%s'." % (applicationName)
  elif not htmMBeans:
    raise Exception("Error: Could not find a TaskManager MBean on %s" % (scope.displayName()))
  else:
    for template in templates:      
      print "     %s '%s', valid from %s, namespace %s, state %s" % (message, template["templateName"], template["validFromString"], template["namespace"], template["state"])
      for htmMBean in htmMBeans:
        AdminControl.invoke(htmMBean, htmMethod, [ template["templateName"], template["validFrom"], template["namespace"], applicationName ] )
      #endfor
      AdminConfig.modify(template["stateManagement"], [ [ "initialState", state ] ])
    #endfor
  #endif

#enddef


#*******************************************************************************************************
# main
#*******************************************************************************************************
# ----------------------
# parse the command line
props = {}
props.update({ "start": "", "stop": "", "uninstall": "" })
props.update({ "force": None })
try:
  props = wsadmin.parse(sys.argv, props)
except ValueError, error:
  usage(str(error))
  sys.exit(1)
#endtry

# -----------------------------
# check command line parameters
if not props["start"] and not props["stop"] and not props["uninstall"]:
  usage("One of the '-start', '-stop', and '-uninstall' parameters must be specified.")
  sys.exit(1)
#endif

if (props["start"] and props["stop"]) or (props["start"] and props["uninstall"]) or (props["stop"] and props["uninstall"]):
  usage("Exactly one of the '-start', '-stop', and '-uninstall' parameters may be specified.")
  sys.exit(1)
#endif

if props["force"] and not props["uninstall"]:
  usage("The '-force' option is only valid for the '-uninstall' parameter.")
  sys.exit(1)
#endif

# ----------------------------------------------------
# make sure the script is running in disconnected mode

if wsadmin.getConnectionStatus() != WsAdmin.CONNECTED:
  if wsadmin.getConnectionStatus() == WsAdmin.DISCONNECTED:
    usage(__file__ + " must run in connected mode, cannot use the '-conntype NONE' option.")
  else:
    print
    print "ERROR: A connection could not be established, aborting."
  #endif
  sys.exit(1)
#endif

# ----------------------------
# execute the desired function
try:
  if props["start"]:
    changeBPCTemplateState(props["start"], "Starting", "startProcessTemplate", "startTaskTemplate", "START")
  elif props["stop"]:
    changeBPCTemplateState(props["stop"], "Stopping", "stopProcessTemplate", "stopTaskTemplate", "STOP")
  elif props["uninstall"]:
    if not props["force"]:
      # As of WPS 6.2, uninstall implicitly stops templates
      AdminApp.uninstall(props["uninstall"])
    else:
      bfmMethodName = "stopProcessTemplateAndDeleteInstancesForced"
      htmMethodName = "stopTaskTemplateAndDeleteInstancesForced"
      changeBPCTemplateState(props["uninstall"], "Stopping", bfmMethodName, htmMethodName, "STOP")
      AdminApp.uninstall(props["uninstall"])
    #endif
  #endif
except:
  AdminConfig.reset()
  printException()
else:
  AdminConfig.save()
#endtry
